package com.watabou.newquay;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.LightingColorFilter;
import android.graphics.Paint;
import android.graphics.PointF;
import android.graphics.Rect;

import com.watabou.newquay.utils.Film;

public class BitmapText extends Visual {

	public String text;
	public Font font;
	
	public PointF scale;
	
	public Paint paint;
	
	public BitmapText( Font font ) {
		this( "", font );
	}
	
	public BitmapText( String text, Font font ) {
		super( 0, 0, 0, 0 );
		
		this.text = text;
		this.font = font;
		
		scale = new PointF( 1, 1 );
	}
	
	@Override
	public void draw() {
		super.draw();
		
		Camera c = camera();
		
		float x = this.x - c.scroll.x * parallax;
		float y = this.y - c.scroll.y * parallax;
		
		Canvas canvas = c.canvas;
		
		if (angle == 0 && scale.x == 1 && scale.y == 1) {
			
			draw( canvas, (int)x, (int)y );
			
		} else {
			
			canvas.save();
			
			canvas.translate( x, y );
			canvas.scale( scale.x, scale.y );
			canvas.rotate( angle );
			draw( canvas, 0, 0 );
			
			canvas.restore();
			
		}
	}
	
	protected void draw( Canvas canvas, int x, int y ) {
		
		int length = text.length();
		Rect dstRect = new Rect();
		
		for (int i=0; i < length; i++) {
			Rect srcRect = font.get( text.charAt( i ) );
			dstRect.set( srcRect );
			dstRect.offsetTo( x, y );
			canvas.drawBitmap( font.bitmap, srcRect, dstRect, paint );
			x += srcRect.width() + font.tracking;
		}
	}
	
	public void measure() {
		
		int length = text.length();
		width = 0;
		height = 0;
		
		for (int i=0; i < length; i++) {
			Rect srcRect = font.get( text.charAt( i ) );
			width += srcRect.width() + (i > 0 ? font.tracking : 0);
			height = Math.max( height, srcRect.height() );
		}
	}
	
	public float width() {
		return width * scale.x;
	}
	
	public float height() {
		return height * scale.y;
	}
	
	public void setColor( int color ) {
		if (color != 0) {
			paint = new Paint();
			paint.setColorFilter( new LightingColorFilter( color, 0 ) );
		} else {
			paint = null;
		}
	}
	
	public static Image createImage( String text, Font font, int color ) {
		
		BitmapText txt = new BitmapText( text, font );
		txt.setColor( color );
		txt.measure();
		
		Bitmap bmp = Bitmap.createBitmap( (int)txt.width, (int)txt.height, font.bitmap.getConfig() );
		txt.draw( new Canvas( bmp ), 0, 0 );
		
		return new Image( bmp );
	}
	
	public static class Font extends Film {
		
		public int tracking = 0;		
		public boolean autoUppercase = false;
		
		public static final String LATIN_UPPER = 
			" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ";
		
		public static final String LATIN_FULL = 
			" !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}";
		
		protected Font( Bitmap bmp ) {
			super( bmp );
		}
		
		public Font( Bitmap bmp, int width, String chars ) {
			this( bmp, width, bmp.getHeight(), chars );
		}
		
		public Font( Bitmap bmp, int width, int height, String chars ) {
			super( bmp );
			
			autoUppercase = chars.equals( LATIN_UPPER );
			
			int bmpWidth = bmp.getWidth();
			int length = chars.length();
			
			int left = 0;
			int top = 0;
			int bottom = height;
			
			for (int i=0; i < length; i++) {
				Rect rect = new Rect( left, top, left += width, bottom );
				add( chars.charAt( i ), rect );
				if (left >= bmpWidth) {
					left = 0;
					top = bottom;
					bottom += height;
				}
			}
		}
		
		protected void splitBy( int color, String chars ) {
			
			autoUppercase = chars.equals( LATIN_UPPER );
			int length = chars.length();
			
			int width = bitmap.getWidth();
			int height = bitmap.getHeight();
			
			int pos;
			
		spaceMeasuring:
			for (pos=0; pos <  width; pos++) {
				for (int j=0; j < height; j++) {
					if (bitmap.getPixel( pos, j ) != color) {
						break spaceMeasuring;
					}
				}
			}
			add( ' ', new Rect( 0, 0, pos, 1 ) );
			
			for (int i=0; i < length; i++) {
				
				char ch = chars.charAt( i );
				if (ch == ' ') {
					continue;
				} else {
					
					boolean found;
					int separator = pos;
					
					do {
						if (++separator >= width) {
							break;
						}
						found = true;
						for (int j=0; j < height; j++) {
							if (bitmap.getPixel( separator, j ) != color) {
								found = false;
								break;
							}
						}
					} while (!found);
					
					add( ch, new Rect( pos, 0, separator, height ) );
					pos = separator + 1;
				}
			}
		}
		
		public static Font colorMarked( Bitmap bmp, int color, String chars ) {
			Font font = new Font( bmp );
			font.splitBy( color, chars );
			return font;
		}
		
		public Rect get( char ch ) {
			return super.get( autoUppercase ? Character.toUpperCase( ch ) : ch );
		}
	}
}
